# Generated by Django 3.2.15 on 2022-09-02 14:08

from django.db import migrations, models

import drc_cmis.validators


class Migration(migrations.Migration):
    dependencies = [
        ("drc_cmis", "0016_alter_cmisconfig_time_zone"),
    ]

    operations = [
        migrations.AddField(
            model_name="cmisconfig",
            name="verzoek_folder_path",
            field=models.CharField(
                default="/DRC/Verzoeken/{{ year }}/{{ month }}/{{ day }}/{{ verzoek }}/",
                help_text="The path where documents related to verzoeken are saved.",
                max_length=500,
                validators=[drc_cmis.validators.verzoek_folder_path_validator],
            ),
        ),
    ]
